%% Exercise 07: Rotation of Closed Polygon Lists for Solid Geometry Design
% 2014-11-26: Tim C. Lueth, MIMED - Technische Universität München, Germany
% (URL: <http://www.mimed.de>)
%% 1. Summary of the already explained aspects of the VLFL-Toolbox 
% In examples 1-6 we've learned about:
%%
% * _Point lists_ (PL), _Vertex lists_ (VL), _Facet lists_ (VL), _Edge lists_ (EL) 
% * Import, and export of _STL-File_ data, Conversion of text strings into solid volumes
% * _Closed polygon lists_ (CPL) for boolesche operations of contours
% * 2.5D extrusion of closed polyogon list (CPL) to _Solid Geometries_ (SG)
% * Spatial transformation, positioning, alignment and merging of solid geometry sets
%
%% 2. List of functions used in this example
% As we learned in example 2 and 4, it is possible to extrude a planar
% point list (PL) or closed polygon (CPL) list into a 2.5D solid geometry. Now, we
% will rotate a CPL around th z-axis. In this case, we consider the CPL or
% the PL always as a x/z-list. Using closed polygon lists, we have to
% remember that before extruding them or rotating them it is necessary to
% guarantee that the outer contour has a counter-clockwise order (ccw).
%%
% In this example, some new functions are introduced:
%%
% * CPLplot to draw the closed polygon list in the x/y plane.
% * PLELofCPL to draw the direction, starting point and end point.
% * CPLuniteCPL to unite several CPL into one and adapt their original directions.
% * SGofCPLrot to rotate a contour around the z-axis
%
%% 3. Rotation of closed polygon lists (CPL)
% For the rotation of a simple contour we use the following functions

%%
% * *CPLplot* to draw the closed polygon list in the x/y plane.
% * *PLELofCPL* to draw the direction, starting point and end point.
% * *SGofCPLrot* to rotate a contour around the z-axis

%% 
% *Exercise: Create a simple point list that touches the y-axis*
close all;
CPL=[0 0; 20 0; 20 10; 0 10];       % Create a simple rectangle (ccw)
CPLplot(CPL);                       % plot the rectangle 
PLELofCPL(CPL);                     % show edges and directions

%%
% *Exercise: Rotate the point list around the z-axis to create a cylinder*
SG=SGofCPLrot(CPL);                 % Solid contour rotation
SGplot(SG);                         % show the solid

%% 
% *Exercise: Create a simple point list with distance to the y-axis*
close all;
CPL=[0 0; 20 0; 20 10; 0 10];       % Create a simple rectangle (ccw)
CPL(:,1)=CPL(:,1)+10;               % shift by 1 on the x-axis
CPLplot(CPL);                       % plot the rectangle 
PLELofCPL(CPL);                     % show edges and directions

%%
% *Exercise: Rotate the point list around the z-axis to create a hollow cylinder*
SG=SGofCPLrot(CPL);                 % Solid contour rotation
SGplot(SG,'b');    

%%
% *Exercise: Some other examples for massiv rotational symetric solids*
SG=SGofCPLrot([0 0; 5 5; 0 10]);                   % Solid contour rotation
subplot(2,2,1); view (-30,30); SGplot(SG,'c'); VLFLplotlight (1,0.9);   
SG=SGofCPLrot([0 0; 4 0;  5 1; 5 9; 4 10; 0 10;]); % Solid contour rotation
subplot(2,2,2); view (-30,30); SGplot(SG,'c'); VLFLplotlight (1,0.9); 
SG=SGofCPLrot([0 -2; 6 -2; 6 0;  5 1; 5 9; 4 10; 0 10;]); % Solid contour rotation
subplot(2,2,3); view (-30,30); SGplot(SG,'c'); VLFLplotlight (1,0.9); 
SG=SGofCPLrot([0 -4; 8 -4; 8 -2; 5 -2; 4 -1; 4 0;  5 1; 5 9; 4 10; 0 10;]); % Solid contour rotation
subplot(2,2,4); view (-30,30); SGplot(SG,'c'); VLFLplotlight (1,0.9); 


%%
% The warnings 'Removed n(m) facets' can be ignored. These warning appear 
% if a part of the contour touches or crosses the x=0 line (y-axis). 
%%
% *Exercise: Creating a bold and a sleeve*

closeall;
r=2; H=40; R=10;

PL=PLcircseg (r,[],0,pi/2);  CPL=PLtransP(PL,[R-r,H-r]);
PL=PLcircseg (r,[],-pi/2,0); CPL=[CPL;0 H; 0 0;PLtransP(PL,[R-r,r])];
SG=SGofCPLrot(CPL);                 % Solid contour rotation

subplot(2,2,1); SGplot(SG,'c'); VLFLplotlight (1,0.9); view (-30,30);
subplot(2,2,3); PLELofCPL (CPL); 

PL=PLcircseg (r,[],0,pi/2);  CPL=PLtransP(PL,[R-r,H-r]);
PL=PLcircseg (r,[],-pi/2,0); CPL=[CPL;PLtransP(PL,[R-r,r])];
SG=SGofCPLrot(CPL); 
subplot(2,2,2); SGplot(SG,'c'); VLFLplotlight (1,0.9); view (-30,30); 
subplot(2,2,4); PLELofCPL (CPL); 

%% 4. Creating spheres by rotating half-circles

%%
% *Exercise: Creating a full sphere*

close all;
PL=PLcircle(10);

VLFLfigure; view(-30,30); grid on;
SG=SGofCPLrot(PL);
SGplot(SG); VLFLplotlight (0,0.5);

%%
% *Exercise: Creating a 8 by 8 sphere*
close all;
PL=PLcircle(10,8);

VLFLfigure; view(-30,30); grid on;
SG=SGofCPLrot(PL,8);
SGplot(SG); VLFLplotlight (0,0.5);

%%
% *Exercise: Creating a half sphere*

close all;
PL=[PLcircseg(10,[],-pi/2,0); 0 0];

VLFLfigure; view(-30,30); grid on;
SG=SGofCPLrot(PL);
SGplot(SG); VLFLplotlight (0,0.5);


%% 5. Creating embedded contours

%%
CPL=[0 0; 20 0; 20 10; 0 10; NaN NaN; 1 1; 9 1; 9 9; 1 9; NaN NaN; 2 2; 8 2; 8 8; 2 8 ];

close all;PLELofCPL(CPL);

%%
CPL=CPLuniteCPL(CPL);
close all; PLELofCPL(CPL);

%%
CPL=flip(CPL);
close all; PLELofCPL(CPL);

%% 6. Rotate Contours around the z-axis
% 
VLFLfigure; view(-30,30); grid on;
SG=SGofCPLrot(CPL);
SGplot(SG); VLFLplotlight (0,0.5);

%%
VLFLfigure; view(-30,30); grid on;
CPL(:,1)=CPL(:,1)+10;
SG=SGofCPLrot(CPL);
SGplot(SG); VLFLplotlight (1,0.5);


%%
VLFLfigure; view(-30,30); grid on;
CPL=PLcircle(30); CPL(:,2)=CPL(:,2)*2;
SG=SGofCPLrot(CPL);
SGplot(SG); VLFLplotlight (1,0.5);

%%
VLFLfigure; view(-30,30); grid on;
CPL=PLcircle(30); CPL(:,2)=CPL(:,2)*2; 
CPL=[CPL;NaN NaN;CPL*0.5];
SG=SGofCPLrot(CPL);
SGplot(SG); VLFLplotlight (0,0.5);

%%
VLFLfigure; view(-30,30); grid on;
CPL=PLcircle(30); CPL(:,2)=CPL(:,2)*2; 

SG=SGcat(SGofCPLrot(CPL),SGswap(SGofCPLrot(CPL*0.5)));
SGplot(SG); VLFLplotlight (0,0.5);

%% 7. Samples of 3D Design
% 
close all
SGsample;

%% Final remarks on toolbox version and execution date
VLFLlicense
%%
% * _Tim Lueth, tested and compiled on OSX 10.7.5 with Matlab 2014b on 2014-11-30_ 
% * _Mattias Traeger, executed and published on 64 Bit PC using Windows with Matlab 2014b on YYYY-MMM-DD_ 
