%% Exercise 08: Slicing, Closing, Cutting and Separation of Solid Geometries
% 2015-06-08: Tim C. Lueth, MIMED - Technische Universität München, Germany
% (URL: <http://www.mimed.de>)
%% 1. Summary of the already explained aspects of the VLFL-Toolbox 
% In examples 1-7 we've learned about:
%%
% * _Point lists_ (PL), _Vertex lists_ (VL), _Facet lists_ (VL), _Edge lists_ (EL) 
% * Import, and export of _STL-File_ data, Conversion of text strings into solid volumes
% * _Closed polygon lists_ (CPL) for boolesche operations of contours
% * 2.5D extrusion of closed polyogon list (CPL) to create _Solid Geometries_ (SG)
% * 2.5D rotation of closed polyogon list (CPL) to create _Solid Geometries_ (SG)
% * Spatial transformation, positioning, alignment and merging of solid geometry sets

%% 2. Create a sample solid for this exercise
% Using the function SGsample it is possible to create samples for an
% experiment, to see all of them or to select one.
close all
SGsample(7);
A=SGsample(7);

%% 3. Analyze a slice plane through a solid geoemtry
% Slicing at a specified z-coordinates is a more complex procedure than
% expected if several solids are processed that can penetrate each other.
% By slicing a single solid, the crossed facets/triangles are separated into 2
% upper and lower parts that will lead to 2 lower and 1 upper facets or 1 
% lower and 2 upper facets depending on how many edges are above or under 
% the cutting plane. For slicing we use the function *SGslicer*.
% Be aware that it is not possible to slice surfaces without crossing edges
% (i.e. surfaces in the z_max or z_min plane)

SGslicer (A,9); 
view (10,30);

%%
% It is also possible just to show the cutting edges of the cutting contour
VLFLfigure;
TR2=SGslicer (A,9); 
VLELplots(TR2.Points, TR2.Constraints);

%%
% The result of the slicing process is a delaunay triangulation of the
% cutting plane. It can be used as cover for closing the cutted solids.
in=isInterior(TR2);
VLFLplots(TR2.Points, TR2.ConnectivityList(in,:),'c');

%%
% *Often we want directly getting a closed contour of a slice.*
CPLofSGslice(A,9); [CPL,warn]=CPLofSGslice(A,9); warn
%%
% *The output parameter warns if a ambiguous cutting result exists*
CPLofSGslice(A,10); [CPL,warn]=CPLofSGslice(A,10); warn

%% 4. Cutting and separating a solid geometries in two parts
% By using the output of SGslicer it is possible to create an upper and
% lower part of an object or even by two cutting plane to cut a part out 
% of a larger obect. This is done by the function *SGcut*.
VLFLfigure;
SGcut(A,9);

%% 
% The next figure shows a separation of the two part by moving the upper
% part upwards.
[L,U]=SGcut(A,9)
VLFLfigure;
SGplot(SGtransP(L,[0 0 -3]),'w');
SGplot(SGtransP(U,[0 0 +3]),'m');
view (50,20);

%% 5. Cutting as useful tool for the ending of complex shaped geoemtries
% Some geometries such as screwnuts have specific geometries that have
% their origin in the manufacturing process of the threads. To create also
% similar shapes it is necessary to create a longer thread and to cut out
% the required length later:
VLFLfigure;
SGthread (10,10,[],[],'C'); view (-30,30);
% [A,b,c]=SGthread (10,10);

%% 
% Now create a longer thread and cut out the required length later.
VLFLfigure;
A=SGthread (10,10+5+5,[],[],'C');
[~,B]=SGcut(A,[5.05 14.95]); B=SGtransP (B,[0 0 -5]);
SGplot(B,'m'); view (-30,30);

%%

%% Final remarks on toolbox version and execution date
VLFLlicense
%%
% * _Tim Lueth, tested and compiled on OSX 10.7.5 with Matlab 2014b on 2015-06-08_ 
% * _Christina Friedrich, executed and published on 64 Bit PC using Windows with Matlab 2015a on 2015-06-17_ 
