%% Exercise 10: Packaging of Sets of Solid Geometries (SG)
% 2015-06-08: Tim C. Lueth, MIMED - Technische Universität München, Germany
% (URL: <http://www.mimed.de>)
%% 1. Summary of the already explained aspects of the VLFL-Toolbox 
% In examples 1-9 we've learned about:
%%
% * _Point lists_ (PL), _Vertex lists_ (VL), _Facet lists_ (VL), _Edge lists_ (EL) 
% * Functions to import, check, and export of _STL-File_ data
% * Functions for generating text strings as solid volumes
% * _Closed polygon lists_ (CPL) for boolesche operations of contours
% * 2.5D extrusion of closed polyogon list (CPL) to _Solid Geometries_ (SG)
% * Spatial transformation and merging of solid geometry sets
% * Relative positioning of solids geometries and Boolean operations on Solids 

%% 2. The four-bar-linkage kit as example for a set of multiple solids
% A very interesting mechanism in mechanics is the four-bar-linkage. It
% consists of four bars that are linked together by 4 rotatorial joints.
% Such a mechanism can be built by 4 different elememts
%%
% # Bar: The basic mechanic link
% # Bolt: A simple bolt that allows rotation
% # Shaft: A simple shaft that transfer torque
% # Spacer: A simple element that is required to achieve parallel bars
close all;
fourBarLinkage (25,40,30,40);	

%%
fourBarLinkageKit ('Bar',40);

%%
fourBarLinkageKit ('Bolt');
%%
fourBarLinkageKit ('Shaft');

%%
fourBarLinkageKit ('Spacer');

%% 3. Packaging a set of solid geometries in a volume
% For a four-bar-linkage we need 4 bars and 4 bolts and may be 2 spacer
% and 2 shafts. For this purpose there is one function

%%
% * *SGpacking* arranges several solid geometries side by side in a volume
close all;
A=fourBarLinkageKit ('Bar',40);
B=fourBarLinkageKit ('Bolt');
C=fourBarLinkageKit ('Shaft');
D=fourBarLinkageKit ('Spacer');
SG=SGpacking({A,B,C,D});
SGplot(SG); view (-30,30); VLFLplotlight (1,0.9); zoompatch;

%%
% Similar it is possible to pack several objects of the same kind into the
% volume and also to define the dimensions of the packing volume. Typically
% the z-coordiante of the volume specification is unlimited or much bigger
% than the xy-coordinats.
close all;
SG=SGpacking({A,A,A,A,B,B,B,B,C,C,D,D},[50,50,1000]);
SGplot(SG); view (-30,30); VLFLplotlight (1,0.9); zoompatch;

%% 4. Using container/collections insted of itemizing the solids
% In many cases we are not interested to list the items in the source code
% but to create a structure containing all objects we want to pack later
% Therefor, we need a data structure that allows to collect several
% solids into something like a container. This can be done by the following
% functions:

%%
% * *SGCaddSG*  Add a single solid geometry to a collection
% * *SGCaddSGn* Add multiple copies of a single solid geometry to a collection
close all
SGC=[];                                            % Create a Solid Geometry Collection
SGC=SGCaddSGn(SGC,fourBarLinkageKit ('Bolt'),20);   % Add 20 bolt to the container
SGC=SGCaddSGn(SGC,fourBarLinkageKit ('Shaft'),20);  % Add 20 shafts to the container

SG=SGpacking(SGC,[30, 30 ,1000]);                   % SGpacking accepts also SGC structs
SGplot(SG); view (-30,30); VLFLplotlight (1,0.9); zoompatch;

%% 5. Create boxes around the packed solids for the final 3D print job
% To handle the print job in a convinient way, it makes sense to create a
% box around the parts and also to write on top of the cover the content or
% the intended use of the box plus may by a date.
close all;
SGboxing(SG,[],[],'.\nTest for Packaging and Boxing\n.');
view (-30,30); VLFLplotlight (1,0.9); zoompatch;

%% 6. Create the four-bar-linkage kit as print job 
close all;
SGC=[];
SGC=SGCaddSGn(SGC,fourBarLinkageKit ('Bar',25),2);
SGC=SGCaddSGn(SGC,fourBarLinkageKit ('Bar',35),2);
SGC=SGCaddSGn(SGC,fourBarLinkageKit ('Bar',40),4);
SGC=SGCaddSGn(SGC,fourBarLinkageKit ('Bolt'),4); 
SGC=SGCaddSGn(SGC,fourBarLinkageKit ('Shaft'),4);
SGC=SGCaddSGn(SGC,fourBarLinkageKit ('Spacer'),4); 
SGA=SGpacking(SGC,[55, 60 ,100]); 
SGB=SGboxing(SGA,[],[],'.\nTim Lueth''s Linkage Kit\n.');
VLFLfigure(SGA); SGplot(SGB,'g');
SG=SGcat(SGA,SGB); view (-130,30); VLFLplotlight (1,0.9); zoompatch;
SGwriteSTL(SG,'EXP10: Four-Bar-Linkage-Kit');
%%

%% Final remarks on toolbox version and execution date
VLFLlicense
%%
% * _Tim Lueth, tested and compiled on OSX 10.7.5 with Matlab 2014b on 2015-06-08_ 
% * _Christina Friedrich, executed and published on 64 Bit PC using Windows with Matlab 2015a on 2015-06-17_ 

