%% Exercise 12: Define Robot Kinematics and Detect Collisions
% 2015-08-09: Tim C. Lueth, MIMED - Technische Universität München, Germany
% (URL: <http://www.mimed.de>)
%% 1. Summary of the already explained aspects of the VLFL-Toolbox 
% In examples 1-11 we've learned about:
%%
% * _Point lists_ (PL), _Vertex lists_ (VL), _Facet lists_ (VL), _Edge lists_ (EL) 
% * Functions to import, check, and export of _STL-File_ data
% * Functions for generating text strings as solid volumes
% * _Closed polygon lists_ (CPL) for boolesche operations of contours
% * 2.5D extrusion of closed polyogon list (CPL) to _Solid Geometries_ (SG)
% * Spatial transformation and merging of solid geometry sets
% * Relative positioning of solids geometries and boolean operations on solids 
% * Packaging and boxing of several parts into a complete assemble kit
% * Attaching coordinates frames for creation of Kinematik Models

%% 2. Loading the 5 components of a 4DoF robot solid model as last time
% Before explaining how to create the parts of a robot kinematik we just
% load such components in. The command line load AIM_robot
clear all; close all; SGfigure;
load ('AIM_SGrobot')
SG0=SGfixerrors(SG0,1e-3); SGchecker(SG0);
SG1=SGfixerrors(SG1,1e-3); SGchecker(SG1);
SG2=SGfixerrors(SG2,1e-3); SGchecker(SG2);
SG3=SGfixerrors(SG3,1e-3); SGchecker(SG3);
SG4=SGfixerrors(SG4,1e-3); SGchecker(SG4);
% save ('AIM_SGrobot','SG0','SG1','SG2','SG3','SG4','SGrobot');
VLFLplotlight(1,0.8);

%% 3. Automatic creation of a the robot 
KMofSGs({SG0,SG1,SG2,SG3,SG4}); 

%% 3. Collision in the joint by the resolution of the surfaces
[KM,XVL]=KMofSGs({SG0,SG1,SG2,SG3,SG4},[],0.05);
KMplot(KM,'m'); VLFLplotlight (1,0.9);

%%
if ~isempty(XVL); zoompatch(XVL); VLplot(XVL,'k*',10);  end;

%% 4. Showing a different robot
KMofSGs({SG0,SG2,SG2,SG1,SG4}); 
view(-30,30);

%% 5. Showing a self collision of a robot
KMofSGs({SG0,SG1,SG2,SG3,SG4},155); 
view(-185,35); VLFLplotlight(1,0.8);
%%

%% Final remarks on toolbox version and execution date
VLFLlicense
%%
% * _Tim Lueth, tested and compiled on OSX 10.7.5 with Matlab 2014b on 2015-06-08_ 
% * ___________, executed and published on 64 Bit PC using Windows with Matlab 2015a on 2015-xx-xx_ 


