%% Exercise 13: Mounting Faces and Conversion of Blocks into Leightweight-structures
% 2015-09-11: Tim C. Lueth, MIMED - Technische Universität München, Germany
% (URL: <http://www.mimed.de>)
%% 1. Summary of the already explained aspects of the VLFL-Toolbox 
% In examples 1-12 we've learned about:
%%
% * _Point lists_ (PL), _Vertex lists_ (VL), _Facet lists_ (VL), _Edge lists_ (EL) 
% * Functions to import, check, and export of _STL-File_ data
% * Functions for generating text strings as solid volumes
% * _Closed polygon lists_ (CPL) for boolesche operations of contours
% * 2.5D extrusion of closed polyogon list (CPL) to _Solid Geometries_ (SG)
% * Spatial transformation and merging of solid geometry sets
% * Relative positioning of solids geometries and boolean operations on solids 
% * Packaging and boxing of several parts into a complete assemble kit
% * Attaching coordinates frames and the creation of kinematik models

%% 2. Analyzing mouting faces of flat surfaces
% All planar faces of a solid can be considered as mounting faces for
% different design purpses. It is useful to calculate or to handle them
% using the following functions:
%%
% * MLofSG - creates the mounting faces and calculates normal vectors and sizes
% * MLplot - plots the mounting faces in different colors
%
%%
% The following example shows the separation of a solid into a set of
% mouting faces wich are represented by a number and a correlation list
% between triangle faces and mounting faces.
close all; SGfigure; view (-30,30);
[ML,MA,SG]=MLofSG(SGbox([60,40,20]));
MLplot(SG);
%%
% * ML defines for all entries of FL the corresponding mounting face.
% * In this example, 12 faces are ordered to 6 mounting faces
ML
%%
% * MA describes for each mounting face, the number, the size, and the normal vector.
% * In this example, we see 6 faces with different normal vectors and sizes
MA
%%
% * SG is a struct of VL and FL extended by ML and MA
SG

%% 3. Analyzing mouting faces of spherical/freeform surfaces
% The concepts of mounting faces supports also spherical mounting faces. In case of spherical
% mounting faces the length of the normal vector is shorter than 1!. This
% information can be used to distinguish between planar and
% spherical or freeformed mounting faces
close all; SGfigure; view (-30,30);
load AIM_SGrobot
[ML,MA,SG]=MLofSG(SG2);
MLplot(SG); 

%% 
% *Now we plot only the two half-spherical mounting faces 5 and 1 of the robot link*
close all; SGfigure; view (-30,30);
MA
MLplot(SG,5); % bended surface, since length of normal vector is less than 1
MLplot(SG,1); % planar surface since length of normal vector is 1
z1=MA(12,3:5) % normal vector of the cylindric surface 1
z2=MA(14,3:5) % normal vector of the cylindric surface 2
%% 4. Create corresponding surfaces parallel to mounting faces
% It is useful to create correspondig surface parallel to mounting faces,
% which can be smaller or larger than the original one. In the next example
% it is shown how to create a parallel surface in distance 5mm for a planar
% surface (#1) and a spherical surface (#5).
%%
% * VLtransN(VL,FL,shrink, distance) - helps to create corresonding surfaces
VLFLplotlight(1,0.8); view(-40,30);
[VL,~,~,FL]=VLtransN(SG.VL,SG.FL(ML==5,:),0,2);
VLFLplot(VL,FL,'y');

[VL,~,~,FL]=VLtransN(SG.VL,SG.FL(ML==1,:),0,5);
VLFLplot(VL,FL,'c');

%% 5. Create solids using the parallel surfaces and a plate thickness
% Often we want to create a plate solid parallel to the mounting face.
%%
% * SGofSurface(VL,FL,thickness, distance, streching) - creates solids parallel to mounting faces.
close all; SGfigure; view (-30,30);
SGplot(SG); VLFLplotlight(1,0.5); view(-40,30);
SG2=SGofSurface(SG.VL,SG.FL(ML==2,:),1,3);
SGplot(SG2,'m');
SG2=SGofSurface(SG.VL,SG.FL(ML==5,:),1,3);
SGplot(SG2,'m');

%% 6. Finding the 2D CPL of a planar 3D Surface
% Many procedures are based on the manipulation of CPL contours.
% Nevertheless not all planar surfacer are in the xy-plane. Therefor, there
% is a function that creates a CPL contour of a surface and returns also
% the transformation matrix for the back transformation.
%%
% * [PL,T]=PLofVLFL(VL,FL) - returns a PL and a transformation matrix
% * [CPL,T]=CPLofVLFL(VL,FL) - returns a CPL and a transformation matrix
close all; SGfigure; view (-30,30);
SGplot(SG); VLFLplotlight(1,0.5); view(-40,30);
[VL,~,~,FL]=VLtransN(SG.VL,SG.FL(ML==2,:),0,10);
VLFLplot(VL,FL);

%%
% *Now show simply the isolated CPL of this mounting face*
close all; SGfigure; view (-30,30); axis on; grid on;
[CPL,T]=CPLofVLFL(VL,FL);
CPLplot(CPL,'b.-',2);
plotT(T);

%% 7. Replace a solid block by covering plates
% By converting a solidblock into a hollow structure using covering plates,
% the weight an mass inertia of a solid is reduced.
%%
% * SGplatesofSGML(SG,thickness) - convert a solid into a plate structure
% * SGweight (SG,sepecific weight,resolution) - slowly calculates the weight 
close all; SGfigure; view (-30,30);

SGN=SGplatesofSGML(SGbox([60,40,20]),1.5);
SGplot(SGN,'w'); VLFLplotlight(1,0.2); 
%%
SGweight(SGbox([60,40,20]),[],1);
%%
SGweight(SGN,[],1);


%% 8. Replace a solid block by covering plates with punched contours
%%
% * SGplatesofSGML(SG,thickness,CPL) - convert a solid into a punched plate structure
close all; SGfigure; view (-30,30);
SGN=SGplatesofSGML(SGbox([60,40,20]),1.5,PLcircle(4));
SGplot(SGN,'w'); VLFLplotlight(1,0.2); 
%%
SGweight(SGN,[],1);
%%

%% Final remarks on toolbox version and execution date
VLFLlicense
%%
% * _Tim Lueth, tested and compiled on OSX 10.7.5 with Matlab 2014b on 2015-09-11_ 
% * ___________, executed and published on 64 Bit PC using Windows with Matlab 2015a on 2015-xx-xx_ 


