%% Exercise 14: Manipulation Functions for Closed Polygons and Laser Cutting (SVG) 
% 2015-09-20: Tim C. Lueth, MIMED - Technische Universität München, Germany
% (URL: <http://www.mimed.de>)
%% 1. Summary of the already explained aspects of the VLFL-Toolbox 
% In examples 1-13 we've learned about:
%%
% * _Point lists_ (PL), _Vertex lists_ (VL), _Facet lists_ (VL), _Edge lists_ (EL) 
% * Functions to import, check, and export of _STL-File_ data
% * Functions for generating text strings as solid volumes
% * _Closed polygon lists_ (CPL) for boolesche operations of contours
% * 2.5D extrusion of closed polyogon list (CPL) to _Solid Geometries_ (SG)
% * Spatial transformation and merging of solid geometry sets
% * Relative positioning of solids geometries and boolean operations on solids 
% * Packaging and boxing of several parts into a complete assemble kit
% * Attaching coordinates frames and the creation of kinematik models
% * Function for mounting faces of solids and creation of lightweight-solids

%% 2. Fill a contour with copies of pattern
% As it was shown already in the function SGplatesofSGML, it often makes
% sense to fill a contour with another pattern. This can be done by using
% one of the following functions:

%%
% * *CPLfillPattern(CPLA, CPLB,w,d)* - fills a contour CPLA with copies of  
% the pattern CPLB with a distance to the outer contour w and distance between the patterns of d 
SGfigure; view(0,90); axis on;
CPLfillPattern(PLsquare(10,10),PLcircle(1),1);

%% 
% *This can also be done with cutted pattern instead of complete pattern*
SGfigure; view(0,90); axis on;
CPLfillPattern(PLsquare(10,10),PLcircle(1),1,[],true);

%% 3. Writing a contour as SVG-File for laser-cutting
% Especially for laser cutting or platting of contours, the SVG file-format
% is very popular. Handling of SVG Files is possible using the following
% functions:
%%
% * *CPLwriteSVG (CPL,Filename)* - writes the contours in a SVG-File
SGfigure; view(0,90); axis on;
A=CPLfillPattern(PLsquare(10,10),PLcircle(1),1,[],true);
CPLplot(A);
CPLwriteSVG(A,'VLFL_EXP14');

%% 4. Calculating the normal vectors of edges and points
SGfigure; view(0,90); 
CPLedgeNormal(PLsquare(10,10)); axis on;

%% 5. Growing with same number of points
SGfigure; view(0,90); 
CPLgrow(PLstar(10,10),1); axis on;

%% 6. Growing with correct distance to edges
SGfigure; view(0,90); axis on;
CPLgrowEdge(PLstar(10,10),1); 

%%
% *Another example using CPLsample*
SGfigure; view(0,90); 
CPLgrowEdge(CPLsample(12),1); axis on;

%% 7. Rounded edges inside a contour
% Another method to change the shape of a contour is to round the edges.
SGfigure; view(0,90); 
PLradialEdges(PLstar(10,10));axis on;
%%
% *Another example using radius=2*
SGfigure; view(0,90); axis on;
PLradialEdges(PLstar(10,10),2); axis on;

%% 8. Sort CPLs around its center
%
% *Find the minmal angle value of a star*
SGfigure; view(0,90); 
CPLsortC(PLstar(10,10),'min'); axis on;
%%
% *Find the minmal angle value of a spiral*
SGfigure; view(0,90); 
CPLsortC(CPLspiral(10,20,4*pi+pi/2),'min');

%%
% *Find the minmal angle value of convex hull of a spiral*
SGfigure; view(0,90); 
CPLsortC(CPLspiral(10,20,4*pi+pi/2),'cmin');

%%
% *Find the maximum angle value of a star*
SGfigure; view(0,90); 
CPLsortC(PLstar(10,10),'max'); axis on;
%%
% *Find the angle value nearest to zero of a star*
SGfigure; view(0,90); 
CPLsortC(PLtrans(PLstar(10,10),rotdeg(160)),'zero'); axis on;

%%
% *Find the angle value nearest to zero of a spiral*
SGfigure; view(0,90); 
CPLsortC(CPLspiral(10,20,4*pi+pi/2),'zero');

%%
% *Find the angle value nearest to zero of convex hull of a spiral*
SGfigure; view(0,90); 
CPLsortC(CPLspiral(10,20,4*pi+pi/2),'czero');


%% 9. Informations on contours inside of others
%
SGfigure; view(0,90); 
CPLinsideCPL(PLcircle(9),CPLcopypattern(PLcircle(2),[5 5],[2 2])); axis on;

%%
% *Identical contours are not inside each other*
CPLinsideCPL(PLcircle(14),CPLsortC(PLcircle(14)));

%% 10. Order contours for the sequential plot with a laser cutter
%
SGfigure; view(0,90); 

%%

%% Final remarks on toolbox version and execution date
VLFLlicense
%%
% * _Tim Lueth, tested and compiled on OSX 10.7.5 with Matlab 2014b on 2015-09-20_ 
% * ___________, executed and published on 64 Bit PC using Windows with Matlab 2015a on 2015-xx-xx_ 


