%% Exercise 20: Programmatically Interface to SimMechanics Multi-Body Toolbox
% 2016-11-19: Tim C. Lueth, MIMED - Technische Universität München, Germany
% (URL: <http://www.mimed.de>)
%% 1. Summary of the already explained aspects of the VLFL-Toolbox 
% *The previous examples of this series covered the following topics:*
%%
% * Example 01: First Steps Using the VLFL-Toolbox for Solid Object Design
% * Example 02: Using the VLFL-Toolbox for STL-File Export and Import
% * Example 03: Closed 2D Contours and Boolean Operations in 2D
% * Example 04: 2.5D Design Using Boolean Operators on Closed Polygon Lists (CPL)
% * Example 05: Creation, Relative Positioning and Merging of Solid Geometries (SG)
% * Example 06: Relative Positioning and Alignment of Solid Geometries (SG)
% * Example 07: Rotation of Closed Polygon Lists for Solid Geometry Design
% * Example 08: Slicing, Closing, Cutting and Separation of Solid Geometries
% * Example 09: Boolean Operations with Solid Geometries
% * Example 10: Packaging of Sets of Solid Geometries (SG)
% * Example 11: Attaching Coordinates Frames to Create Kinematik Models
% * Example 12: Define Robot Kinematics and Detect Collisions
% * Example 13: Mounting Faces and Conversion of Blocks into Leightweight-structures
% * Example 14: Manipulation Functions for Closed Polygons and Laser Cutting (SVG)
% * Example 15: Create a Solid by 2 Closed Polygons
% * Example 16: Create a Solid along a contour
% * Example 20: Programmatically Interface to SimMechanics Multi-Body Toolbox

%% 2. Creating a new SimMechanics System
smbNewSystem ('SG_LIB_EXP_20');           % Creates the mechansim diagramm  
smbDrawNow;

%% 3. Create two links with length 50 and 80 and one or two mounting holes 
SG1=SGmodelLink(80,'',1,2);               % Creates a long rod with flange
SG2=SGmodelLink(50,'',1,2);               % Creates a short rod with flange

%%
SGfigure(SG1); view(-30,30);

%%
SGfigure(SG2); view(-30,30);

%% 4. Create SimMechanics models for the four links in different colors
smbCreateSG (SG1,'LINK1','r');              % Add long rod as LINK1 
smbCreateSG (SG2,'LINK2','g');              % Add short rod as LINK2 
smbCreateSG (SG1,'LINK3','y');              % Add long rod as LINK3
smbCreateSG (SG2,'LINK4','c');              % Add short rod as LINK4 
smbDrawNow;

%% 5. Create SimMechanics models for the four joint and connect them with the links
smbCreateJoint ('R','R1','LINK1.F','LINK2.B'); % Add a RR Joint
smbCreateJoint ('R','R2','LINK2.F','LINK3.B'); % Add a RR Joint
smbCreateJoint ('R','R3','LINK3.F','LINK4.B'); % Add a RR Joint
smbCreateJoint ('R','R4','LINK4.F','LINK1.B'); % Add a RR Joint
smbDrawNow;

%% 6. Connect the base frame of link1 to the world coordinate system
smbCreateConnection('WORLD.ORIGIN','LINK1.B'); % Connect Linkage to World Frame
smbDrawNow;

%% 7. Run the Simulation of the Simulink/SimMechanics diagram for 1 second
smbSimulate(1);         % Simulate for 1 second

%% 8. Create a Video of the Simualatin for 5 seconds
smbVideoSimulation(5);  % Show a 5 seconds video

%% Final remarks on toolbox version and execution date
VLFLlicense
%%
% * _Tim Lueth, tested and compiled on OSX 10.11.6 with Matlab 2016 on 2016-12-09_ 
% * ___________, executed and published on 64 Bit PC using Windows with Matlab 2015a on 2015-xx-xx_ 

