%% Exercise 22: Adding Simulink Signals to Record Frame Movements
% 2016-12-18: Tim C. Lueth, MIMED - Technische Universität München, Germany
% (URL: <http://www.mimed.de>)
%% 1. Summary of the already explained aspects of the VLFL-Toolbox 
% *The previous examples of this series covered the following topics:*
%%
% * Example 01: First Steps Using the VLFL-Toolbox for Solid Object Design
% * Example 02: Using the VLFL-Toolbox for STL-File Export and Import
% * Example 03: Closed 2D Contours and Boolean Operations in 2D
% * Example 04: 2.5D Design Using Boolean Operators on Closed Polygon Lists (CPL)
% * Example 05: Creation, Relative Positioning and Merging of Solid Geometries (SG)
% * Example 06: Relative Positioning and Alignment of Solid Geometries (SG)
% * Example 07: Rotation of Closed Polygon Lists for Solid Geometry Design
% * Example 08: Slicing, Closing, Cutting and Separation of Solid Geometries
% * Example 09: Boolean Operations with Solid Geometries
% * Example 10: Packaging of Sets of Solid Geometries (SG)
% * Example 11: Attaching Coordinates Frames to Create Kinematik Models
% * Example 12: Define Robot Kinematics and Detect Collisions
% * Example 13: Mounting Faces and Conversion of Blocks into Leightweight-structures
% * Example 14: Manipulation Functions for Closed Polygons and Laser Cutting (SVG)
% * Example 15: Create a Solid by 2 Closed Polygons
% * Example 16: Create a Solid along a contour
% * Example 20: Programmaticaly Interface to SimMechanics Multi-Body Toolbox
% * Example 21: Programmaticaly Design of SimMechanisms Mechanisms and Simulink Signals
% * Example 22: Adding Simulink Signals to Record Frame Movements

%% 2. Creating a new SimMechanics System
smbNewSystem ('SG_LIB_EXP_22');           % Creates the mechansim diagramm  
smbDrawNow;

%% 3. Create two links with length 50 and 80 and one or two mounting holes 
SG1=SGmodelLink(80,'',1,2);               % Creates a long rod with flange
SG2=SGmodelLink(50,'',1,2);               % Creates a short rod with flange

%% 4. Create SimMechanics models for the four links and four joints in different colors
smbCreateSG (SG1,'LINK1','r');              % Add long rod as LINK1 
smbCreateSG (SG2,'LINK2','g');              % Add short rod as LINK2 
smbCreateSG (SG1,'LINK3','y');              % Add long rod as LINK3
smbCreateSG (SG2,'LINK4','c');              % Add short rod as LINK4 
smbCreateJoint ('R','R1','LINK1.F','LINK2.B'); % Add a RR Joint
smbCreateJoint ('R','R2','LINK2.F','LINK3.B'); % Add a RR Joint
smbCreateJoint ('R','R3','LINK3.F','LINK4.B'); % Add a RR Joint
smbCreateJoint ('R','R4','LINK4.F','LINK1.B'); % Add a RR Joint
smbCreateConnection('WORLD.ORIGIN','LINK1.B'); % Connect Linkage to World Frame
smbDrawNow;

%% 5. Create a video of the movements
% smbVideoSimulation(3);  % Show a 3 seconds video

%% 6. Analyze the simulation for 3 Seconds
% The result of a simulation is a strucutre that contains SimMultiBody
% states (xout) and recorded Simulink signals (sim). If there are no
% Simulink signals, sout is empty.
simOut=smbSimulate(3)
%%
% The states contain the parameter = angles/velocity of the joints
xout = simOut.get('xout')
%%
% There is no Simulink signals yet
sout = simOut.get('sout')

%% 7. Create Simulink signals for all the frames of the four links
smbAddFrameSensor ('LINK1.RF');
smbAddFrameSensor ('LINK2.RF');
smbAddFrameSensor ('LINK3.RF');
smbAddFrameSensor ('LINK4.RF');

%%
% Now, all links have simulink signals and signal output for R and T of the
% reference frame
smbDrawNow;

%%
% The model of link4 is extendend by a transformation sensor
smbDrawNow ('LINK4.RF_T');

%% 8. Simulate and record those signals too
simOut=smbSimulate(3)
smbVideoSimulation(3);
%%
% The states contain the parameter = angles/velocity of the joints
xout = simOut.get('xout')
%%
% TheSimulink signals are related to the reference rotation and translation
sout = simOut.get('sout')
T1=smbTofSimOut(simOut,'LINK1.RF'); VL1=squeeze(T1(1:3,4,:))';
T2=smbTofSimOut(simOut,'LINK2.RF'); VL2=squeeze(T2(1:3,4,:))';
T3=smbTofSimOut(simOut,'LINK3.RF'); VL3=squeeze(T3(1:3,4,:))';
T4=smbTofSimOut(simOut,'LINK4.RF'); VL4=squeeze(T4(1:3,4,:))';
SGfigure; axis on
VLplot(VL1,'r.-');
VLplot(VL2,'g.-');
VLplot(VL3,'y.-');
VLplot(VL4,'c.-');
drawnow;

%% Final remarks on toolbox version and execution date
VLFLlicense
%%
% * _Tim Lueth, tested and compiled on OSX 10.11.6 with Matlab 2016b on 2016-12-18_ 
% * ___________, executed and published on 64 Bit PC using Windows with Matlab 2015a on 2015-xx-xx_ 

